﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

using _3dstate;

/*
 * 
 *  How to write a 3D application
 *  ------------------------------
 *  
 *  Copy this example or if you want to start from scratch do:
 * 
 *  1) Create a new empty project ( C# Windows Forms Application)
 *     - Add the _3D.cs file to your project 
 *     - Add:   using  _3dstate;  (at the top of any files using the 3dstate 3D engine)
 *     - Allow unsafe code (the 3dstate 3D engine is a win32 DLL therefor one need to allow unsafe code)
 *       it doesnt mean that it is not safe ... only that win32 dlls are not .NET managed code
 *       (project properties -> Build Tab)
 *  
 *  2) Add the Form1_Load event and the Form1_FormClosing event
 *  
 *      Open the Form (double click on Form1.cs) , open the properties window -> Events 
 *      double click on the Load event and on the FormClosing events
 *      copypadte the code from this page to the two events
 *      Note that we dont have to load the world on the Form1_Load event
 *      we can load the world whenever we want to.
 *      Closing the engine though, should be the last thing we do (after that we cannot use the 3D engine anymore)
 *      
 * 
 *  3) - Add a pictureBox control to our form. We will render (put the 3D image) on this control
 *       note that we can also use other controls such as buttons, the form itself etc.
 *     - Anchor the pictureBox control to the Form. (on the properties of the control select the Anchor property and mark all 4 sides i.e Top, Bottom, Left, Right)
 *       Anchoring all 4 sides ensures that when the window is resized the target rendering control will also be resized.
 *       Of course if you want the rendering image to be a "fixed size" then you can skip this step and sticks with the default anchoring (Top, Left)
 *       
 *  4)  Add a timer or an Idle event.
 *
 *      to add a timer do:
 *      ------------------
 *         Timer m_timer; //define as a class member so when the Form is closing we can stop the timer
 *         
 *         //in the Form1_Load() or whenever we want to start rendering the 3D world do:
 *         m_timer = new Timer();
 *         m_timer.Interval = 10; //call every 10 milliseconds
 *         m_timer.Tick += new EventHandler(timer_Tick); //the timer_Tick is the function that will be called
 *         m_timer.Start();
 *         
 *      add the tmer function:
 *      
 *      void timer_Tick(object sender, EventArgs e)
 *      {
 *
 *            IntPtr window_handle = pictureBox_rendering_window.Handle;
 *           
 *            //this is the function that puts the 3D image on the screen
 *            _3D.STATE_engine_render(window_handle, 0);
 *
 *      }
 *      
 *  4B) instead of a timer we can use the Idel event.
 *      to do so:
 *      
 *         1)   EventHandler m_idleEventHandler; //define is a member of the class so that when the Form1_Closing() we can do Application.Idle -= m_idleEventHandler;
 *
 *         2)   //in the Form1_Load() or whenever we want to start rendering the 3D world do:
 *            m_idleEventHandler = new System.EventHandler(OnIdle);
 *            Application.Idle += m_idleEventHandler;
 *
 *         3)  //add this function
 *           
 *              void OnIdle(object sender, EventArgs e)
 *              {
 *                  IntPtr window_handle = pictureBox_rendering_window.Handle;
 *                  _3D.STATE_engine_render(window_handle, 0);
 *              }
 *
 * 
 * 

 */


namespace Hello_3D_World22
{
    public partial class Form1 : Form
    {
        private EventHandler m_idleEventHandler;

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {

            int was_loaded_ok = _3D.STATE_engine_load_world("../../../../worlds/trainYard/trainyard.3dstate", "", "", _3D.USER_DEFINED_BEHAVIOR);
            if (was_loaded_ok != _3D.OK)
            {
                MessageBox.Show("Failed to load world", "Failed to load world", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }


            m_idleEventHandler = new System.EventHandler(OnIdle);
            Application.Idle += m_idleEventHandler;


        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            Application.Idle -= m_idleEventHandler;
        }

        //We use the onIdle event to do the rendering.
        //The advantage is that the rendinging only takes available CPU
        //when the program is busy doing something else, onIdle wont be called.
        //Another advantage is that we work on the same thread as the user interface
        //Alternatively we can use a timer or a separate thread
        private void OnIdle(object sender, EventArgs e)
        {

            //the STATE_engine_render() is the most important function in the 3D engine
            //it takes a picture of the 3D world (as seen from the current view point/camera)
            //and put it on the given window
            IntPtr window_handle = pictureBox_rendering_window.Handle;
            _3D.STATE_engine_render(window_handle, 0); //0 means we use the default camera as the view point


            //do something ... lets turn the camera
            uint camera = _3D.STATE_camera_get_default_camera();
            _3D.STATE_camera_rotate_z(camera, 0.4, _3D.WORLD_SPACE);


            //we need the code below so that OnIdle will be called again
            //the code below creates an OnPaint event which when finish calls OnIdle
            Rectangle rect = new Rectangle(0, 0, 0, 0);
            this.Invalidate(rect, false);


        }


    }
}
